/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package algo;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringTokenizer;

import node.BlatAntNode;
import support.NetworkSystem;
import support.Parameters;
import support.Randomizer;
import ants.AntSystem;
import ants.BalancingAnt;
import ants.DiscoveryAnt;

public class BlatantRAlgorithm extends Algorithm {

	public BlatantRAlgorithm(String[] args) {
		setupCLI(args);
		parseCLI();

		System.out.format("---------------------------------------------\n");
		System.out.println("Blatant-R Algorithm Implementation");
		System.out.println("(C) 2008 Amos Brocco <amos.brocco@unifr.ch>");
		System.out.format("---------------------------------------------\n");

		printConfiguration();
		init();
		importYaml(Parameters.INITIAL);
		System.out.format("NODES = %d\n", NetworkSystem.getNodeCount());
		System.out.format("---------------------------------------------\n");
		
		this.execute();
	}

	private boolean active = true;

	private boolean firstActive() {
		if (active) {
			active = false;
			return true;
		}
		return false;
	}

	protected synchronized void addNode(String id, String hook) {
		if (!NetworkSystem.existNode(id)) {
			BlatAntNode node = new BlatAntNode(new String(id), firstActive());
			NetworkSystem.addNode(node);
			//Stats.newNode();
		}
		
		if (!hook.trim().equals("")) {
			NetworkSystem.initiateUserLink(new String(id), new String(hook));
		}
	}

	public static void main(String[] args) {
		new BlatantRAlgorithm(args);
	}

	protected void generateAnts() {
		// AntFactory.preallocate(1000);
		for (String n : NetworkSystem.getNodes()) {
			generateAnt(n);
			if (Parameters.BALANCING) {
				if (NetworkSystem.getNode(n).getLoadManagementService()
						.getLoad() > 50)
					AntSystem.schedule(new BalancingAnt(n));
			}
			// AntSystem.schedule(new DAnt(n));
		}
	}

	public void generateAnt(String n) {
		if (Randomizer.pop() < Parameters.ANT_BIRTH_PROBABILITY) {
			if (!Parameters.DISABLE_BLATANT)
				AntSystem.schedule(new DiscoveryAnt(n));
		}
	}

	private void importYaml(String path) {

		String line = null;
		String currentNode = null;
		BufferedReader br = null;

		try {
			br = new BufferedReader(new FileReader(path));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

		try {
			while ((line = br.readLine()) != null) {
				if (line.charAt(0) != ' ') {
					// Skip this
				} else {
					StringTokenizer st = new StringTokenizer(line, ":");
					String id = new String(st.nextToken().trim());
					if (id.equals("datastructure"))
						break;
					if (st.countTokens() > 0) {
						// Neighbor
						if (!NetworkSystem.existNode(id))
							NetworkSystem.addNode(new BlatAntNode(
									new String(id), firstActive()));
						NetworkSystem.initiateUserLink(new String(id), new String(currentNode));
					} else {
						// New node
						if (!NetworkSystem.existNode(id))
							NetworkSystem.addNode(new BlatAntNode(
									new String(id), firstActive()));
						currentNode = id;
					}
				}
			}
			br.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	protected void algorithmStep() {
	}

}
