#!/usr/bin/python
import sys
from operator import itemgetter


def isSearchResult(line):
	try:
		if line[0:2] == ";;":
			return True
		else:
			return False
	except:
		return False

def printClassment(dictionary, hightolow = False, limit = 20):
	classment = sorted(dictionary.items(), key=itemgetter(1))
	if (hightolow):
		classment.reverse()
	for t in classment[:limit]:
		print "\t",t[0],":",t[1]


class GenericLine():
	def __init__ (self, line):
		try:
			data = line.strip().split("\t")
			self.iter = int(data[0])
			self.tokens = int(data[6].split("/")[1][:-1])
			self.antpings = int(data[6].split("(")[1].split("/")[0])
		except:
			print line
			sys.exit(1)

class ProactiveMergesLine():
	def __init__ (self, line):
		try:
			data = line.strip().split("\t")[0].split("/")
			self.tokens = int(data[2].split("=")[1])
			self.hops = int(data[3].split("=")[1])
			self.replies =int(data[5].split("=")[1])
			self.pings = int(data[7].split("=")[1][:-1])
		except:
			print line
			sys.exit(1)

def isProactiveMergesLine(line):
	try:
		if (line.strip()[0] == "|"):
			return True
		else:
			return False
	except:
		return False

def isGenericLine(line):
	try:
		if (line.count("S/L/X") > 0):
			return True
		else:
			return False
	except:
		return False
		

class ResultLine():
	def __init__ (self, line):
		data = line.split(" ")
		self.iter = data[1]
		r = data[2].split('\t')
		self.valid =  line.split('\t')[4].split('/')[3].split("=")[1].split("(")[0] != "NaN"
		self.queries = int(r[0].split("=")[1])
		self.ttl = int(r[1].split("/")[0].split("=")[1])
		self.ffw = int(r[1].split("/")[1].split("=")[1])
		self.success = int(r[2].split("/")[0].split("=")[1])
		self.poolsize = int(r[2].split("/")[1].split("=")[1])
		self.hits = int(r[2].split("/")[2].split("=")[1])
		self.hitscache = int(r[2].split("/")[3].split("=")[1].split("(")[1][:-1])
		self.hops = int(r[3].split("=")[1].split("(")[0])
		self.hopscache = int(r[3].split("=")[1].split("(")[1][:-1])


for startIteration in [25000]:
	for endIteration in [76000]:
		if startIteration >= endIteration:
			continue

		byhitrate = {}
		bysuccessrate = {}
		byaveragehops = {}
		byhopsperhit = {}
		byhitcacherate = {}
		byaveragehopscache = {}
		byestimatedcostperhit = {}
		byestimatedcostperquery = {}


		for resultFile in sys.argv[1:]:

			print "-"*80

			print "Data file:", resultFile
			f = open(resultFile,'r')
			contents = f.readlines()
			f.close()

			totalQueries = 0
			totalSuccess = 0
			totalPoolSize = 0
			totalHits = 0
			totalHitsCache = 0
			totalHops = 0
			totalTokens = 0
			totalPings = 0
			totalReplies = 0
			totalHopsCache = 0
			totalTokens = 0
			estimatedCost = 0
			totalAntPings = 0
			
			HOPCOST = 1024 + 4
			PINGCOST = 64 + 4
			TOKENCOST = 308 + 4
			REPLYCOST = 320 + 4
			ANTPINGSCOST = 288 + 4

			first = None
			last = None

			lastIter = -1

			for line in contents:
				if isGenericLine(line):
					k = GenericLine(line)
					lastIter = int(k.iter)
					if (int(k.iter) >= startIteration) and (int(k.iter) <= endIteration):
						totalAntPings += k.antpings
						totalTokens += k.tokens - k.antpings
				elif isSearchResult(line):
					k = ResultLine(line)
					if (int(k.iter) >= startIteration) and (int(k.iter) <= endIteration):
						if (k.valid):
							if first is None:
								first = int(k.iter)
							totalQueries += k.queries
							totalSuccess += k.success
							totalPoolSize += k.poolsize
							totalHits += k.hits
							totalHitsCache += k.hitscache
							totalHops += k.hops
							totalHopsCache += k.hopscache
							last = k.iter
				elif isProactiveMergesLine(line):
					if (lastIter >= startIteration) and (lastIter <= endIteration):
						k = ProactiveMergesLine(line)
						totalHops += k.hops
						totalPings += k.pings
						totalTokens += k.tokens
						totalReplies += k.replies
					

			estimatedCost += HOPCOST * totalHops + PINGCOST * totalPings + TOKENCOST * totalTokens + REPLYCOST * totalReplies + ANTPINGSCOST * totalAntPings
			successRate = float(totalSuccess) / totalQueries * 100
			hitRate = float(totalHits) / totalPoolSize * 100
			hitCacheRate = float(totalHitsCache) / totalHits * 100
			averageHops = float(totalHops) / totalQueries
			averageHopsCache = float(totalHopsCache) / totalQueries
			averageHopsPerHit = float(totalHops) / totalHits

			averageTokens = float(totalTokens) / totalQueries
			averagePings = float(totalPings) / totalQueries
			averageReplies = float(totalReplies) / totalQueries
			averageCost = float(estimatedCost) / totalQueries
			averageCostPerHit = float(estimatedCost) / totalHits
			averageCostPerQuery = float(estimatedCost) / totalQueries

			byhitrate[resultFile] = hitRate
			byhitcacherate[resultFile] = hitCacheRate
			bysuccessrate[resultFile] = successRate
			byaveragehops[resultFile] = averageHops
			byaveragehopscache[resultFile] = averageHopsCache
			byhopsperhit[resultFile] = averageHopsPerHit
			byestimatedcostperhit[resultFile] = averageCostPerHit
			byestimatedcostperquery[resultFile] = averageCostPerQuery

			print "From iteration", first,"to",last
			print "\tTotal queries:", totalQueries
			print "\tTotal success:", totalSuccess
			print "\tTotal pool size:", totalPoolSize
			print "\tTotal hits:", totalHits
			print "\tof which from cache:", totalHitsCache
			print "\tTotal hops:", totalHops
			print "\tTotal tokens:", totalTokens
			print "\tTotal pings:", totalPings
			print "\tTotal replies:", totalReplies
			print "\tEstimated cost:", estimatedCost
			print "\tof which from cache:", totalHopsCache
			print "\tSuccess rate %:", successRate
			print "\tHit rate % (hits vs pool):", hitRate
			print "\tof which in cache % (cache hits vs hits):", hitCacheRate
			print "\tAverage hops (total):", averageHops
			print "\tof which in cache:", averageHopsCache
			print "\tAverage tokens:", averageTokens
			print "\tAverage pings:", averagePings
			print "\tAverage replies:", averageReplies
			print "\tAverage cost:", averageCost
			print "\tAverage hops per hit:", averageHopsPerHit
			print "\tAverage cost per hit:", averageCostPerHit
			print "\tAverage cost per query:", averageCostPerQuery
			print "-"*80

			print "_"*80


		print
		print "By hit rate:"
		printClassment(byhitrate, hightolow = True)
		print
		print "By Success rate:"
		printClassment(bysuccessrate, hightolow = True)
		print
		print "By average hops:"
		printClassment(byaveragehops)
		print
		print "By hops per hit:"
		printClassment(byhopsperhit)
		print
		print "By hits from cache rate:"
		printClassment(byhitcacherate, hightolow = True)
		print
		print "By hops in cache:"
		printClassment(byaveragehopscache, hightolow = True)
		print		
		print "By estimated average cost per hit:"
		printClassment(byestimatedcostperhit)
		print
		print "By estimated average cost per query:"
		printClassment(byestimatedcostperquery)
		


