/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package ants;

import services.dss.DSSStats;
import services.dss.JobStatusManager.Status;
import support.NetworkSystem;

public class DSSStatusAnt extends Ant {

	private String target = null;
	private int step = 0;
	private String jobid;
	private String assignee;
	private Status status;
	
	private long ert = -1;
	private long drift = -1;
	private Double timestamp = -1.0;

	public DSSStatusAnt(String id, String target, String jobid, Status s, String assignee, Double timestamp) {
		super(id);
		this.target = target;
		this.jobid = jobid;
		this.assignee = assignee;
		this.status = s;
		this.timestamp  = timestamp;
		DSSStats.status(jobid, assignee, s, target);
	}

	/* Constructor for simulation only */
	public DSSStatusAnt(String identifier, String target, String jobid,
			Status s, String assignee, Double timestamp, long remainingTime, long drift) {
		this(identifier, target, jobid, s, assignee, timestamp);
		this.ert = remainingTime;
		this.drift = drift;
	}

	public void antstep() {
		if (!NetworkSystem.isNodeAlive(current_)) {
			// We are on a dead node
			kill();
			return;
		}
		switch (step) {
		case 0:
			if (!migrate(target)) {
				step = 4;
				kill();
				return;
			} else {
				DSSStats.statusHop();
				step = 1;
			}
			return;
		case 1:
			current.acquire();
			if (ert != -1) { // Simulation only
				current.getDSSService().getScheduler().getProfile(jobid).remainingTime = ert;
				current.getDSSService().getScheduler().getProfile(jobid).drift = drift;
			}
			current.getDSSService().status(jobid, assignee, status, timestamp);
			current.release();
			step = 4;
			kill();
			return;
		default:
		}
	}

	@Override
	public void statmigration() {
	}

}
