/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  BeBop Asynchronous Communication Library
 *  
 *  Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 *  					Pervasive and Artificial Intelligence Research Group
 *  					Department of Informatics DIUF
 *  					University of Fribourg, Fribourg
 *  					Switzerland
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package bebop.client;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;

import bebop.lib.IMessage;
import bebop.lib.Message;
import bebop.serialization.ISerializer;
import bebop.serialization.Serializer;

public class Client {
	
	private static int MAX_MESSAGE_SIZE = Short.MAX_VALUE;
	
	private static List<SendAction> actions = new ArrayList<SendAction>();
	
	static class SendAction {
		public IMessage msg;
		public String host;
		public int port;
		
		public SendAction(String host, int port, IMessage msg) {
			this.msg = msg;
			this.host = host;
			this.port = port;
		}

		public SendAction(String host, Message msg) {
			this.host = host;
			this.msg = msg;
			this.port = 7855;
		}
	}

	static class PostmanThread implements Runnable {

		public void run() {
			for (;;) {
				try {
					synchronized(actions) {
						for (SendAction sa : actions) {
							sendMessageNow(sa.host, sa.msg);
						}
						actions.clear();
					}
					Thread.sleep(10);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	public static void init() {
		new Thread(new PostmanThread()).start();
	}
	

	public static void sendMessageNow(String address, IMessage m)
			throws Exception {
		if (address.startsWith("bebop://")) {
			address = address.replace("bebop://", "");
			String host = "localhost";
			int port = 7855;
			if (address.contains(":")) {
				String[] splitted = address.split(":");
				host = splitted[0];
				splitted = splitted[1].split("/");
				port = Integer.parseInt(splitted[0]);
			} else {
				String[] splitted = address.split("/");
				host = splitted[0];
			}
			sendMessageNow(host, port, m);
		}
	}

	private static void sendMessageNow(String host, int port, IMessage m) throws IOException {
		Socket s = null;
		ByteArrayOutputStream out = null;
		OutputStream oStream = null;
		try {
			s = new Socket(InetAddress.getByName(host), port);
			s.setKeepAlive(true);
			s.setSoTimeout(0);
			
			byte[] buffer = new byte[MAX_MESSAGE_SIZE];
			for (int k = 0; k < MAX_MESSAGE_SIZE; k++)
				buffer[k] = '\0';
			out = new ByteArrayOutputStream();
			List<Object> l = new ArrayList<Object>();
			l.add(m.getRecipient());
			l.add(m.getContents());
			ISerializer ser = new Serializer(out);
			ser.print(l);
			String ba = out.toString();
			oStream = new BufferedOutputStream(s.getOutputStream());
			oStream.write(ba.getBytes());
			oStream.write('\n');
			oStream.flush();
			oStream.close();
			s.close();
		} catch (Exception e){
			if (oStream != null) {
				oStream.close();
			}
			if (out != null) {
				out.close();
			}
			if (s != null) {
				s.close();
			}
		}
	}


	public static void sendMessage(String address, Message msg) {
		SendAction s = new SendAction(address, msg);
		synchronized(actions) {
			actions.add(s);
		}
	}

}
