/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  Solenopsis: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006  Amos Brocco <amos.brocco@unifr.ch>
 *  					Pervasive and Artificial Intelligence Research Group
 *  					Department of Informatics DIUF
 *  					University of Fribourg, Fribourg
 *  					Switzerland
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
package cli;

public class LongOption extends CLIOption<Long> {

	private String description;
	private String shortLabel;
	private String longLabel;
	private long value;
	private long defaultValue;
	private boolean isMandatory;

	public LongOption(String shortL, String longL, String desc, long defValue,
			boolean mandatory) {
		shortLabel = shortL;
		longLabel = longL;
		defaultValue = defValue;
		description = desc;
		value = defaultValue;
		isMandatory = mandatory;
	}

	private boolean isused = false;

	public boolean used() {
		return isused;
	}

	public String getDefault() {
		return new Long(defaultValue).toString();
	}

	public String getDescription() {
		return description;
	}

	public String getLongLabel() {
		return longLabel;
	}

	public String getShortLabel() {
		return shortLabel;
	}

	public int getSkip() {
		return 2;
	}

	public void setValue(String string) {
		value = Long.parseLong(string);
		//System.out.println("Value " + string + " is " + value);
	}

	public Long getValue() {
		isused = true;
		return value;
	}

	public boolean isMandatory() {
		return isMandatory;
	}

	public String getType() {
		return "long";
	}

	public boolean isValidValue(String value) {
		try {
			this.value = Long.parseLong(value);
			return true;
		} catch (NumberFormatException e) {
			return false;
		}
	}

	public boolean ignoreInvalid() {
		return false;
	}

}
