/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package network;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import support.Parameters;
import support.Randomizer;

public class NetworkNodeGen {
	
	private static List<String> nodePool;
	
	public static void init() {
		nodePool = new ArrayList<String>();
		if (RTT.getNodePool() != null) {
			nodePool.addAll(RTT.getNodePool());
			Collections.shuffle(nodePool, Randomizer.getRandom());
		}
	}
	
	public static String grab() {
		if (!Parameters.RTT_MIGRATION)
			return new Integer((int) (Randomizer.pop() * 100000)).toString();
		synchronized(nodePool) {
			if (nodePool.isEmpty() && Parameters.RTT_GENERATOR)
				return new Integer((int) (Randomizer.pop() * 100000)).toString();
			return nodePool.remove(0);
		}
	}
	
	public static void grab(String node) {
		if (!Parameters.RTT_MIGRATION) return;
		synchronized(nodePool) {
			nodePool.remove(node);
		}
	}
	
	public static void release(String node) {
		if (!Parameters.RTT_MIGRATION) return;
		synchronized(nodePool) {
			nodePool.add(node);
		}
	}

}
