/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package network;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;

import support.Parameters;

public class RTTCacheYaml implements IRTT {
	
	private static long[][] weights = null;
	private static int size;
	private Set<String> pool = new HashSet<String>();
	
	public long getPathWeight(String x, String y) {
		if ((x == null) || (y == null)) return 0;
		if (x.equals(y)) return 0;
		if (weights != null) {
			int idx = Integer.parseInt(x) % size;
			int idy = Integer.parseInt(y) % size;
			long value = weights[idx][idy];
			//System.out.format("%d ms are %d iterations\n", weights[idx][idy], value);
			return value;
		} else  {
			System.out.println("No path to");
			return -1;
		}
	}

	public void initRTT(boolean directed, String unetpath) {
		size = getGraphSize(unetpath);
		importYaml(size, unetpath);
	}
	
	private void importYaml(int size, String path) {
		System.out.println("Importing "+path+" ("+Parameters.MS_PER_ITER_DOUBLE+" ms per iter)");
		
		String line = null;
		int currentNode = -1;
		BufferedReader br = null;
		
		weights = new long[size][size];
		
		for (int o=0; o< size; o++)
			Arrays.fill(weights[o], -1);

		try {
			br = new BufferedReader(new FileReader(path));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

		long paths = 0;
		long total = 0;
		long totaliters = 0;
		
		try {
			while ((line = br.readLine()) != null) {
				if (line.charAt(0) != ' ') {
					// Skip this
				} else {
					StringTokenizer st = new StringTokenizer(line, ":");
					String id = new String(st.nextToken().trim());
					int idx = Integer.parseInt(id);
					if (id.equals("datastructure"))
						break;
					if (st.countTokens() > 0) {
						// Neighbor
						Double weight = null;
						if (st.hasMoreTokens()) {
							String wgs = new String(st.nextToken().trim());
							weight = Double.parseDouble(wgs);
						} else {
							weight = 1.0;
						}
						int weighto = weight.intValue();
						long iters =  Math.round(weighto / Parameters.MS_PER_ITER_DOUBLE);
						weights[currentNode][idx] = iters;
						if (currentNode != idx) {
							paths++;
							total += weighto;
							totaliters += iters;
						}
					} else {
						// New node
						pool.add(id);
						currentNode = idx;
					}
				}
			}
			br.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		System.out.format("%d total YAML paths loaded, RTT Factor %d, average roundtrip %d ms (%d iterations)\n", paths, Parameters.MS_PER_ITER, total / paths, Math.round((total/paths) / Parameters.MS_PER_ITER_DOUBLE));
	}
	
	private int getGraphSize(String path) {
		String line = null;
		int nodes = 0;
		BufferedReader br = null;

		try {
			br = new BufferedReader(new FileReader(path));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

		try {
			while ((line = br.readLine()) != null) {
				if (line.charAt(0) != ' ') {
					// Skip this
				} else {
					StringTokenizer st = new StringTokenizer(line, ":");
					String id = new String(st.nextToken().trim());
					if (id.equals("datastructure"))
						break;
					if (st.countTokens() > 0) {
					} else {
						nodes++;
					}
				}
			}
			br.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return nodes;
	}
	
	public long mstoiter(long ms) {
		return ms / Math.round(Parameters.MS_PER_ITER_DOUBLE);
	}
	
	
	public long itertoms(long iter) {
		return iter * Parameters.MS_PER_ITER;
	}

	public Set<String> getNodePool() {
		return pool;
	}
	
	
	public static void main(String[] args) {
		long d = 34;
		long a = 53;
		long b = 95;
		long c = 123;
		double factor = 50;
		
		System.out.println(Math.round(a/factor));
		System.out.println(Math.round(b/factor));
		System.out.println(Math.round(c/factor));
		System.out.println(Math.round(d/factor));
	}
	

}
