/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package node;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import support.Parameters;
import support.Stats;

public class CentralizedNode extends Node {

	private List<String> neighbors;

	public CentralizedNode(String id) {
		super(id);
		neighbors = new ArrayList<String>();
	}



	public boolean userconnect(String target) {
		if (!neighbors.contains(target) && (!target.equals(id))) {
			if (Parameters.NOTIFY)
				System.out.format("Adding %s -> %s\n", id, target);
			Stats.addLink();
			neighbors.add(target);
		}
		return true;
	}

	public void cleanLeave() {
		return;
	}

	public void clearNode() {
		return;
	}

	public boolean connect(String target) {
		return userconnect(target);
	}

	public boolean disconnect(String target) {
		if (neighbors.contains(target)) {
			if (Parameters.NOTIFY)
				System.out.format("Removing %s -> %s\n", id, target);
			Stats.removeLink();
			neighbors.remove(target);
		}
		return true;
	}

	public Collection<String> getNeighbors() {
		Collection<String> ns = new ArrayList<String>(neighbors.size());
		ns.addAll(neighbors);
		return ns;
	}

	public int getNeighborsCount() {
		return neighbors.size();
	}

	public int getNeighborNeighborsSize(String n) {
		return -1;
	}

	public boolean isNeighbor(String source) {
		return neighbors.contains(source);
	}

	@Override
	public void timestep() {
	}
	
	public void iterstep() {
		
	}

}
