/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package services.dss;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import services.dss.profile.ResourceProfile;
import support.INetworkNode;
import support.Parameters;

public class AcceptOfferManager {

	class Offer {
		public Long finishTime = Long.MAX_VALUE;
		public String node = null;
	}

	private long OWT = 0;

	private Map<String, Offer> offers = new HashMap<String, Offer>();
	
	private Map<String, Long> bidEnd = new HashMap<String, Long>();

	private Map<String, ResourceProfile> profiles = new HashMap<String, ResourceProfile>();

	private INetworkNode node;
	
	public AcceptOfferManager(INetworkNode n, long offerWaitTimems) {
		node = n;
		OWT = offerWaitTimems;
	}

	public long previous = 0l;
	
	public void timestep() {
		synchronized (offers) {
			List<String> closedBids = new ArrayList<String>();
			long lt = node.getTime();
			
				previous = lt;
			for (String jobid : bidEnd.keySet()) {
				Long end = bidEnd.get(jobid);
				if (end < lt) {
					closedBids.add(jobid);
				}
			}
			for (String jobid : closedBids) {
				bidEnd.remove(jobid);
				if ((offers.get(jobid) != null) && (offers.get(jobid).node != null)) {
					Offer o = offers.get(jobid);
					DSSStats.assigned(jobid, o.node);
					profiles.get(jobid).promisedCosts.add(0, o.finishTime);
					Messaging.sendAssign(node, profiles.get(jobid), o.node);
				} else {
					DSSStats.failed(jobid, node.getIdentifier());
					ResourceProfile o = profiles.remove(jobid);
					offers.remove(jobid);
					o.submitRetries++;
					if ((Parameters.DSS_MAX_RETRIES < 0) || (o.submitRetries <= Parameters.DSS_MAX_RETRIES)) {
						DSSStats.retry(jobid, node.getIdentifier());
						offers.put(o.id, new Offer());
						profiles.put(o.id, o);
						bidEnd.put(o.id, node.getTime() + OWT);
					}
				}
				stopAcceptOffersFor(jobid);
			}
		}
	}

	public boolean isWaitingJob(String jobid) {
		synchronized (offers) {
			return offers.containsKey(jobid);
		}
	}

	public void pushOffer(String jobid, String node, Long estimatedFinishTime) {
		synchronized (offers) {
			if (isWaitingJob(jobid)) {
				Offer current = offers.get(jobid);
				if ((current.finishTime > estimatedFinishTime)) {
					//System.out.println("Got offer for "+jobid+" from "+node+" in "+estimatedFinishTime+" "+profiles.get(jobid));
					Offer noffer = new Offer();
					noffer.node = node;
					noffer.finishTime = estimatedFinishTime;
					offers.put(jobid, noffer);
				}
			}
		}
	}

	public void startAcceptOffersFor(ResourceProfile job) {
		synchronized (offers) {
			//System.out.println("Start accepting offers for "+job.id+" "+job);
			DSSStats.submitted(job, node.getIdentifier());
			offers.put(job.id, new Offer());
			profiles.put(job.id, job);
			bidEnd.put(job.id, node.getTime() + OWT);
		}
	}

	public void stopAcceptOffersFor(String jobid) {
		synchronized (offers) {
			//System.out.println("Stop accepting offers for "+jobid+" "+profiles.get(jobid));
			offers.remove(jobid);
			profiles.remove(jobid);
			bidEnd.remove(jobid);
		}
	}
	
	public int getAcceptingQueueSize() {
		synchronized (offers) {
			return offers.size();
		}
	}

}
