/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package services.dss;

import java.text.DateFormat;

import services.dss.JobStatusManager.Status;
import services.dss.profile.ResourceProfile;
import support.NetworkSystem;

public class DSSStats {
	
	public static long finished = 0l;
	public static long totalTime = 0l;
	public static long totalWaiting = 0l;
	public static long totalExecuting = 0l;
	public static long totalETA = 0l;
	public static long totalETA1 = 0l;
	
	public static long requestHops = 0l;
	public static long informHops = 0l;
	public static long acceptHops = 0l;
	public static long assignHops = 0l;
	
	public static long requestStarted = 0l;
	public static long informStarted = 0l;
	public static long acceptStarted = 0l;
	public static long assignStarted = 0l;
	
	public static long submitted = 0l;
	public static long failed = 0l;
	
	public static long totalDeadlines = 0l;
	public static long totalMissedDeadlines = 0l;
	public static long totalPositiveSlack = 0l;
	public static long totalMissedSlack = 0l;
	public static int totalReservations = 0;
	public static long totalStartDelayTime = 0l;
	public static int totalMissedReservations = 0;
	public static long totalEndDelayTime = 0l;
	public static long totalEndBonusTime = 0l;
	public static long running = 0l;
	
	public static long revoked = 0l;
	public static long revokedjobs = 0l;
	
	public static long finishedpool = 0l;
	public static long finishedpooljobs = 0l;
	
	
	public static long statusStarted = 0l;
	public static long statusHops = 0l;
	

	
	public synchronized static void submitted() {
		submitted++;
	}
	
	public synchronized static void failed() {
		failed++;
	}
	
	public synchronized static void requestStarted() {
		requestStarted++;
	}
	
	public synchronized static void informStarted() {
		informStarted++;
	}
	
	public synchronized static void acceptStarted() {
		acceptStarted++;
	}
	
	public synchronized static void assignStarted() {
		assignStarted++;
	}
	
	public synchronized static void requestHop() {
		requestHops++;
	}
	
	public synchronized static void informHop() {
		informHops++;
	}
	
	public synchronized static void acceptHop() {
		acceptHops++;
	}
	
	public synchronized static void assignHop() {
		assignHops++;
	}
	
	public synchronized static void liveStats() {
		
		long avTime = 0;
		long avWait = 0;
		long avExecuting = 0;
		long avETA = 0;
		long avETA1 = 0;
		
		if (finished != 0) {
			avTime = totalTime / finished;
			avWait = totalWaiting / finished;
			avExecuting = totalExecuting / finished;
			avETA = totalETA / finished;
			avETA1 = totalETA1 / finished;	
		}
		

	
		System.out.format("[LIVESTATS] -- %d (%s) -- finished jobs %d ; " +
				"average time %s ; average wait %s ; " +
				"average execution %s ; average estimated %s, " +
				"average estimated(1) %s; request %d/%d : accept %d/%d : inform %d/%d : assign %d/%d : status %d/%d\n", NetworkSystem.getGlobalTimeIter(), formatTime(NetworkSystem.getGlobalTimeMs()), 
				finished, formatTime(avTime), formatTime(avWait), formatTime(avExecuting), formatTime(avETA), formatTime(avETA1), requestStarted, requestHops, acceptStarted, acceptHops, informStarted, informHops, assignStarted, assignHops,
				statusStarted, statusHops);
		System.out.format("[DEADLINES] -- Total %d , Missed %d , Positive Slacktime %d , Missed Slacktime %d \n", totalDeadlines, totalMissedDeadlines, totalPositiveSlack, totalMissedSlack);
		System.out.format("[RESERVATIONS] -- Total %d , Missed %d , Bonus %d , Start delay %d, End delay %d \n", totalReservations, totalMissedReservations, totalEndBonusTime, totalStartDelayTime, totalEndDelayTime);
		System.out.format("[POOLS] -- Finished %d, Revoked %d , Finished Jobs %d, Revoked Jobs %d \n", finishedpool, revoked, finishedpooljobs, revokedjobs);
	}
	
	public synchronized static void finished(ResourceProfile job, double cpuspeed) {
		long time = NetworkSystem.getGlobalTimeMs();
		long total = time - job.createdAt; 
		long executing = time - job.startedAt;
		long waiting = total - executing;
		long eta = job.getEstimatedRunningTime(cpuspeed);
		long eta1 = job.getEstimatedRunningTime(1.0);
		System.out.format("[FINISH] -- %d (%s) -- %s ; total %d ; wait %d; execution %d ; estimated %d, estimated(1) %d ; ACost %s / PCost %s ; Informs/Assigns %d / %d \n", 
				NetworkSystem.getGlobalTimeIter(), formatTime(NetworkSystem.getGlobalTimeMs()), job.id, 
				total, waiting, executing, eta, eta1, job.actualCosts.toString(), job.promisedCosts.toString(), job.informs, job.assigns);
		
		finished += 1;
		totalTime += total;
		totalWaiting += waiting;
		totalExecuting += executing;
		totalETA += eta;
		totalETA1 += eta1;
		
		running -= 1;
		
		long avTime = totalTime / finished;
		long avWait = totalWaiting / finished;
		long avExecuting = totalExecuting / finished;
		long avETA = totalETA / finished;
		long avETA1 = totalETA1 / finished;
	
		System.out.format("[ACTUALSTATS] -- %d (%s) -- finished jobs %d ; " +
				"average time %s ; average wait %s ; " +
				"average execution %s ; average estimated %s, " +
				"average estimated(1) %s; request %d/%d : accept %d/%d : inform %d/%d : assign %d/%d\n", NetworkSystem.getGlobalTimeIter(), formatTime(NetworkSystem.getGlobalTimeMs()), 
				finished, formatTime(avTime), formatTime(avWait), formatTime(avExecuting), formatTime(avETA), formatTime(avETA1), requestStarted, requestHops, acceptStarted, acceptHops, informStarted, informHops, assignStarted, assignHops);
	}
	
	public static void status(String jobid, String assignee, Status s, String initiator) {
		System.out.format("[STATUS] -- %d (%s) -- Job %s on node %s set to %s (informing initiator %s)\n", NetworkSystem.getGlobalTimeIter(), formatTime(NetworkSystem.getGlobalTimeMs()), jobid, assignee, s.toString(), initiator);
	}
	
	public static void reassigned(String jobid, String target) {
		System.out.format("[REASSIGN] -- %d (%s) -- %s to %s\n", NetworkSystem.getGlobalTimeIter(), formatTime(NetworkSystem.getGlobalTimeMs()), jobid, target);
	}
	
	public static void inform(String jobid, String node) {
		System.out.format("[INFORM] -- %d (%s) -- %s on %s\n", NetworkSystem.getGlobalTimeIter(), formatTime(NetworkSystem.getGlobalTimeMs()), jobid, node);
	}
	
	public static void submitted(ResourceProfile job, String target) {
		submitted();
		System.out.format("[SUBMIT] -- %d (%s) -- %s to %s\n", NetworkSystem.getGlobalTimeIter(), formatTime(NetworkSystem.getGlobalTimeMs()), job.toString(), target);
	}
	
	public static void failed(String jobid, String target) {
		failed();
		System.out.format("[FAILED] -- %d (%s) -- %s on %s\n", NetworkSystem.getGlobalTimeIter(), formatTime(NetworkSystem.getGlobalTimeMs()), jobid, target);
	}
	
	public static void assigned(String jobid, String target) {
		System.out.format("[ASSIGN] -- %d (%s) -- %s to %s\n", NetworkSystem.getGlobalTimeIter(), formatTime(NetworkSystem.getGlobalTimeMs()), jobid, target);
	}
	
	public static synchronized void started(String jobid) {
		System.out.format("[START] -- %d (%s) -- %s\n", NetworkSystem.getGlobalTimeIter(), formatTime(NetworkSystem.getGlobalTimeMs()), jobid);
		running += 1;
	}
	
	public static String formatTime(long millis) {
		String sign = millis >= 0 ? "+" : "-"; 
		millis = Math.abs(millis);
		long hours = millis / 3600000l;
		if (hours > 1000) {
			
			return DateFormat.getDateTimeInstance(
		            DateFormat.LONG, DateFormat.LONG).format(millis);
		}
		long minutes = (millis - hours * 3600000) / 60000;
		long seconds = (millis - (minutes * 60000 + hours * 3600000)) / 1000;
		long ms = millis - (minutes * 60000 + hours * 3600000+ seconds * 1000);
		return String.format("%s %dh %dm %ds %dms", sign, hours, minutes, seconds, ms);
	}

	public static void retry(String jobid, String target) {
		System.out.format("[RETRY] -- %d (%s) -- %s on %s\n", NetworkSystem.getGlobalTimeIter(), formatTime(NetworkSystem.getGlobalTimeMs()), jobid, target);
	}

	public static synchronized void deadline(long currentTime, Long deadline) {
		totalDeadlines++;
		if (currentTime > deadline) {
			totalMissedDeadlines += 1;
			totalMissedSlack += currentTime - deadline;
			System.out.format("[ACTUALDEADLINE] -- FAILED , Latetime %d \n", currentTime - deadline);
		} else {
			totalPositiveSlack += deadline - currentTime;
			System.out.format("[ACTUALDEADLINE] -- SUCCEED , Slacktime %d \n", deadline - currentTime);
		}
	}

	public static synchronized void reservation(Long currentTime, Long dueStart, Long actualStart, Long dueEnd, Long actualEnd) {
		totalReservations ++;
		long startDelay = actualStart - dueStart;
		Long endDelay = actualEnd - dueEnd;
		totalStartDelayTime += startDelay;
		if (endDelay > 0) {
			totalMissedReservations += 1;
			totalEndDelayTime += endDelay;
			System.out.format("[RESERVATION] -- FAILED , dueStart %d, actualStart %d (delay %d), dueEnd %d, " +
					"actualEnd %d (delay %d) \n", 
					dueStart, actualStart, startDelay, dueEnd, actualEnd, endDelay);
		} else {
			totalEndBonusTime += Math.abs(endDelay);
			System.out.format("[RESERVATION] -- SUCCEED , dueStart %d, actualStart %d (delay %d), dueEnd %d, " +
					"actualEnd %d (delay %d) \n", 
					dueStart, actualStart, startDelay, dueEnd, actualEnd, endDelay);
		}
	}

	public static synchronized void revokePool(String id, int size) {
		System.out.format("[REVOKE] Pool %s , size %d\n", id, size);
		revoked++;
		revokedjobs += size;
	}

	public static synchronized void finishPool(String id, int size) {
		System.out.format("[FINISHPOOL] Pool %s , size %d\n", id, size);
		finishedpool++;
		finishedpooljobs += size;
		
	}

	public static synchronized void statusStarted() {
		statusStarted++;
	}
	
	public static synchronized void statusHop() {
		statusHops++;
	}

}
