/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package services.dss.monitor;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.util.Hashtable;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.UIManager;

import services.dss.DSSStats;
import services.dss.DynamicSchedulingService;
import services.dss.profile.ResourceProfile;
import support.INetworkNode;
import support.NetworkSystem;
import support.Parameters;

public class DSSMonitorStats implements Runnable {
	
	private static Hashtable<String, NodeSchedulingPanel> panels;
	
	private static Container main;
	
	private static JPanel j;
	
	private static JFrame detailWindow;
	
	private static JLabel statsLabel;
	
	private static JLabel detailLabel;
	
	private static volatile ResourceProfile highlightJob = null;
	
	private static long startTime = 0l;
	
	public static void init(Container m) {
		if (main == null) {
			main = m;
			panels = new Hashtable<String, NodeSchedulingPanel>();
			new Thread(new DSSMonitorStats()).start();
		}
	}
	
	public static void init() {
		if (!Parameters.DSS_VISUAL_SCHEDULER) return;
		if (main == null) {
						
			try {
				UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			} catch (Exception e) {
			}
			
			JFrame f = new JFrame("DSS Monitor");
			f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			//
			
			detailWindow = new JFrame("Detail");
			detailWindow.setSize(640, 640);
			JPanel cont = new JPanel();
			detailLabel = new JLabel();
			cont.setLayout(new BorderLayout());
			detailWindow.add(cont);
			cont.add(detailLabel);
			
			f.setSize(800, 600);
			
			JPanel base = new JPanel();
			base.setLayout(new BoxLayout(base, BoxLayout.PAGE_AXIS));
			
			f.add(base);
			
			JPanel statsPanel = new JPanel();
			base.add(statsPanel);
			
			//statsPanel.setMaximumSize(new Dimension(800, 140));
			statsPanel.setMinimumSize(new Dimension(640, 190));
			statsPanel.setPreferredSize(new Dimension(800, 190));
			statsPanel.setSize(new Dimension(800, 190));
			statsPanel.setLayout(new BorderLayout());
			statsLabel = new JLabel();
			statsLabel.setVerticalTextPosition(JLabel.TOP);
			statsLabel.setHorizontalAlignment(SwingConstants.LEADING);
			statsPanel.add(statsLabel);
			
			JScrollPane p = new JScrollPane();
			base.add(p);
			
			j = new JPanel();
			j.setLayout(new BoxLayout(j, BoxLayout.PAGE_AXIS));
			p.add(j);
			p.setViewportView(j);
			f.setVisible(true);
			startTime = System.currentTimeMillis();
			init(j);
			
		}
	}
	
	public static void addDaemon(INetworkNode d, DynamicSchedulingService s) {
		if (!Parameters.DSS_VISUAL_SCHEDULER) return;
		synchronized(panels) {
			NodeSchedulingPanel p = new NodeSchedulingPanel(d, s);
			panels.put(d.getIdentifier(), p);
			main.add(p);
		}
	}
	
	public static void removeDaemon(String identifier) {
		synchronized(panels) {
			NodeSchedulingPanel p = panels.remove(identifier);
			if (p != null) {
				main.remove(p);
			}
		}
	}
	
	public static void toggleDetailWindow() {
		if (detailWindow == null) return;
		detailWindow.setVisible(!detailWindow.isVisible());
	}
	
	public static void setDetailText(String when, String text) {
		if (detailWindow == null) return;
		detailWindow.setTitle("Detail @ "+when);
		detailLabel.setText(text);
		detailLabel.repaint();
	}

	public void run() {
		for(;;) {
			synchronized(panels) {
				
				int totalJobs = 0;
				int totalWaiting = 0;
				long totalScheduleLength = 0;
								
				int nonEmptySchedules = 0;
				
				for (NodeSchedulingPanel p : panels.values()) {
					p.update();
					p.repaint();
					DynamicSchedulingService thedss = p.getDSS();
					if (thedss != null) {
						totalJobs += thedss.getScheduler().getScheduleSize();
						totalWaiting += thedss.getAcceptOfferManager().getAcceptingQueueSize();
						long length = p.getDSS().getScheduler().getScheduleLength();
						totalScheduleLength += length;
						if (length != 0)
							nonEmptySchedules++;
					}
				}
				
				main.repaint();
				
				String realElapsed = DSSStats.formatTime(System.currentTimeMillis() - startTime);
				String simElapsed = DSSStats.formatTime(NetworkSystem.getGlobalTimeMs());
				
				int nodes = panels.size();
				
				long averageSLength = 0;
				if (nonEmptySchedules != 0) {
					averageSLength = nodes == 0 ? 0 : totalScheduleLength / nonEmptySchedules;
				}
				
				long averageSSize = 0;
				if (nonEmptySchedules != 0) {
					averageSSize = nodes == 0 ? 0 : totalJobs / nonEmptySchedules;
				}
				
				long finished = DSSStats.finished;
				
				String avTime = "--";
				String avWait = "--";
				String avExecuting = "--";
				String avETA = "--";
				String avETA1 = "--";
				
				String avStartDelayTime = "--";
				String avEndBonus = "--";
				String avEndDelay = "--";
								
				if (finished != 0) {
					avTime = DSSStats.formatTime(DSSStats.totalTime / finished);
					avWait =  DSSStats.formatTime(DSSStats.totalWaiting / finished);
					avExecuting = DSSStats.formatTime(DSSStats.totalExecuting / finished);
					avETA =  DSSStats.formatTime(DSSStats.totalETA / finished);
					avETA1 =  DSSStats.formatTime(DSSStats.totalETA1 / finished);
					
					avStartDelayTime = DSSStats.formatTime(DSSStats.totalStartDelayTime / finished);
					avEndBonus = DSSStats.formatTime(DSSStats.totalEndBonusTime / finished);
					avEndDelay = DSSStats.formatTime(DSSStats.totalEndDelayTime / finished);
				}
				
				
				statsLabel.setText(String.format("<html><b>Statistics (iter: %d / sim: %s / real: %s)</b>" +
						"<table border=\"0\">" +
							"<tr>" +
								"<td><b>Nodes</b>:</td>" +
									"<td>%d</td>" +
								"<td><b>Scheduled Jobs</b>:</td>" +
								"	<td>%d</td>" +
								"<td><b>Waiting Jobs</b>:</td>" +
								"	<td>%d</td>" +
								"<td><b>Av. Schedule</b>:</td>" +
								"	<td>%s</td>" +
							"</tr>" +
							"<tr>" +
								"<td><b>Request (Hops)</b>:</td>" +
									"<td>%d (%d)</td>" +
								"<td><b>Accept (Hops)</b>:</td>" +
								"	<td>%d (%d)</td>" +
								"<td><b>Assign (Hops)</b>:</td>" +
								"	<td>%d (%d)</td>" +
								"<td><b>Inform (Hops)</b>:</td>" +
								"	<td>%d (%d)</td>" +
							"</tr>" +
						"<tr>" +
							"<td><b>Finished Jobs</b>:</td>" +
								"<td>%d</td>" +
							"<td><b>Av. Total Time</b>:</td>" +
							"	<td>%s</td>" +
							"<td><b>Av. Waiting Time</b>:</td>" +
							"	<td>%s</td>" +
							"<td><b>Av. Execution Time</b>:</td>" +
							"	<td>%s</td>" +
						"</tr>" +
						"<tr>" +
							"<td><b>Av. ETA Time</b>:</td>" +
								"<td>%s</td>" +
							"<td><b>Av. ETA(1)</b>:</td>" +
							"	<td>%s</td>" +
							"<td><b>Submitted</b>:</td>" +
							"	<td>%d</td>" +
							"<td><b>Failed</b>:</td>" +
							"	<td>%d</td>" +
						"</tr>" +
						"<tr>" +
						"<td><b>Idle</b>:</td>" +
							"<td>%d</td>" +
						"<td><b>Non-Idle</b>:</td>" +
						"	<td>%d</td>" +
						"<td><b>Av. Size</b>:</td>" +
						"	<td>%d</td>" +
						"<td><b>Running Tasks</b>:</td>" +
						"	<td>%d</td>" +
						"</tr>" +
						"<tr>" +
						"<td><b>Reservations</b>:</td>" +
							"<td>%d</td>" +
						"<td><b>Missed</b>:</td>" +
						"	<td>%d</td>" +
						"<td><b>Av. Start Delay</b>:</td>" +
						"	<td>%s</td>" +
						"<td><b>Av. End Bonus</b>:</td>" +
						"	<td>%s</td>" +
						"</tr>" +
						"<tr>" +
						"<td><b>Av. End Delay</b>:</td>" +
							"<td>%s</td>" +
						"<td><b>Revoked Pools</b>:</td>" +
						"	<td>%d (%d)</td>" +
						"<td><b>Finished Pools</b>:</td>" +
						"	<td>%d (%d)</td>" +
						"<td><b>Status (Hops)</b>:</td>" +
						"	<td>%d (%d)</td>" +
						"</tr>" +
						"</table>"+
						"</html>", NetworkSystem.getGlobalTimeIter(), simElapsed, realElapsed, 
						nodes, totalJobs, totalWaiting, DSSStats.formatTime(averageSLength),
						DSSStats.requestStarted, DSSStats.requestHops, 
						DSSStats.acceptStarted, DSSStats.acceptHops, 
						DSSStats.assignStarted, DSSStats.assignHops, 
						DSSStats.informStarted, DSSStats.informHops,
						finished, avTime, avWait, avExecuting, 
						avETA, avETA1, DSSStats.submitted, DSSStats.failed,
						nodes - nonEmptySchedules, nonEmptySchedules, averageSSize, DSSStats.running,
						DSSStats.totalReservations, DSSStats.totalMissedReservations,
						avStartDelayTime, avEndBonus, avEndDelay, DSSStats.revoked, DSSStats.revokedjobs, DSSStats.finishedpool, DSSStats.finishedpooljobs,
						DSSStats.statusStarted, DSSStats.statusHops));
				
				if (detailWindow.isVisible() && (highlightJob != null) && (highlightJob.currentNode != null)) {
					setDetailText(simElapsed, SchedulerMonitorWidget.formatTooltip(highlightJob.currentNode.getDSSService().getScheduler(), highlightJob));
				} else {
					setDetailText(simElapsed, "n/a");
				}
			}
			
			
			
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	public static void setHighlightedJob(ResourceProfile job) {
		highlightJob = job;
	}
	
	public static String getHighlightedJobId() {
		if (highlightJob == null) return null;
		return highlightJob.id;
	}
	
	public static ResourceProfile getHighlightedJob() {
		return highlightJob;
	}
	
	private static String highlightPool = null;
	
	public static String getHighlightedPoolId() {
		return highlightPool;
	}
	
	public static void setHighlightedPoolId(String id) {
		highlightPool = id;
	}
		
		

}
