/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package services.dss.profile;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import jsc.distributions.Distribution;
import jsc.distributions.Normal;
import jsc.distributions.Uniform;
import support.NetworkSystem;
import support.Parameters;


public class ProfileGenerator {

	/* Distributions of arch and os adapted from Top500 as per 06/2009 report */
	String[] arch = { "power", "power", "power", "power", "power", "power",
			"power", "power", "power", "power", "power", "power", "power",
			"power", "power", "power", "power", "power", "power", "power",
			"power", "power", "power", "power", "power", "power", "power",
			"power", "power", "power", "power", "power", "power", "power",
			"power", "power", "power", "power", "power", "power", "power",
			"power", "power", "power", "power", "power", "power", "power",
			"power", "power", "power", "power", "power", "power", "power",
			"nec", "sparc", "mips", "ia-64", "ia-64", "ia-64", "ia-64",
			"ia-64", "ia-64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64", "amd64", "amd64", "amd64",
			"amd64", "amd64", "amd64", "amd64" };
	Long[] disk = { 1024l, 2048l, 4096l, 8192l, 16384l };
	private Distribution distribution;
	Long[] memory = { 1024l, 2048l, 4096l, 8192l, 16384l };
	String[] os = { "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "linux", "linux", "linux", "linux",
			"linux", "linux", "linux", "bsd", "windows", "windows", "windows",
			"windows", "windows", "unix", "unix", "unix", "unix", "unix",
			"unix", "unix", "unix", "unix", "unix", "unix", "unix", "unix",
			"unix", "unix", "unix", "unix", "unix", "unix", "unix", "unix",
			"unix", "solaris", "solaris", "solaris", "solaris", "solaris",
			"solaris", "solaris", "solaris", "solaris", "solaris", "solaris",
			"solaris", "solaris", "solaris", "solaris", "solaris", "solaris",
			"solaris", "solaris", "solaris", "solaris", "solaris", "solaris",
			"solaris", "solaris", "solaris", "solaris", "solaris", "solaris" };

	private Distribution csdistribution = new Normal();
	
	private Distribution rtdistribution;
	Long[] version = { 1l };

	public ProfileGenerator(boolean uniform, Distribution pd, Distribution rtd) {
		distribution = pd;
		rtdistribution = rtd;
		dldistribution = getDeadlineDistribution();
		if (uniform) {
			arch = new String[] { "power", "nec", "sparc", "mips", "ia-64", "amd64"};
			disk = new Long[] { 1024l, 2048l, 4096l, 8192l, 16384l };
			memory = new Long[] { 1024l, 2048l, 4096l, 8192l, 16384l };
			os = new String[] { "linux", "bsd", "windows", "unix", "solaris"};
		}
	}

	public ResourceProfile generate(String id) {
		ResourceProfile j = new ResourceProfile();
		j.id = id;
		j.os = os[(int) (distribution.random() * os.length)];
		j.arch = arch[(int) (distribution.random() * arch.length)];
		j.minimumVersion = version[(int) (distribution.random() * version.length)];
		j.memory = memory[(int) (distribution.random() * memory.length)];
		j.disk = disk[(int) (distribution.random() * disk.length)];
		long time = Math.min(Math.max((long) rtdistribution.random(), Parameters.DSS_MIN_JOBTIME_MS), Parameters.DSS_MAX_JOBTIME_MS);
		j.setEstimatedRunningTime(time);
		j.deadline = NetworkSystem.getGlobalTimeMs() + time + Math.min(Math.max((long) dldistribution.random(), Parameters.DSS_MIN_DEADLINE_MS), Parameters.DSS_MAX_DEADLINE_MS);
		
		if (Parameters.NO_CPUSPEED) {
			j.cpuspeed = 1.0f;
		} else {
			j.cpuspeed = Math.min(2.0, Math.max(1.0, csdistribution.random() + 1.5)); // Fixed after ICDCS 2010
		}
		
		j.reservationStart = NetworkSystem.getGlobalTimeMs() + Math.min(Math.max((long) dldistribution.random(), Parameters.DSS_MIN_DEADLINE_MS), Parameters.DSS_MAX_DEADLINE_MS);
		j.reservationEnd = j.reservationStart + time;
		
		j.actualCosts = new ArrayList<Long>();
		j.promisedCosts = new ArrayList<Long>();
		
		j.delays = new ArrayList<Long>();
		return j;
	}
	
	public List<ResourceProfile> generatePool(ResourceProfile base, String poolid, int size) {
		List<ResourceProfile> jobs = new ArrayList<ResourceProfile>();
		System.out.format("[POOL] id %s, Size %d\n", poolid, size);
		base.pool = poolid;
		base.poolsize = size;
		for (int i=0; i < size; i++) {
			ResourceProfile copy = base.copy();
			copy.id = UUID.randomUUID().toString();
			jobs.add(copy);
		}
		return jobs;
	}
	
	private static Distribution dldistribution;
	
	private Distribution getDeadlineDistribution() {
		if (dldistribution == null) {
			double av = (Parameters.DSS_MAX_DEADLINE_MS - Parameters.DSS_MIN_DEADLINE_MS) / 2;
			double stdev = av * 0.5;
			av = Parameters.DSS_MIN_DEADLINE_MS + av;
			dldistribution = new Normal(av, stdev);
		}
		return dldistribution;
	}

	public void setResourcesDistribution(Distribution d) {
		distribution = d;
	}

	public void setRunningTimeDistribution(Distribution d) {
		rtdistribution = d;
	}
	
	public static void main(String[] args) {
		double av = (Parameters.DSS_MAX_JOBTIME_MS - Parameters.DSS_MIN_JOBTIME_MS) / 2;
		double stdev = av * 0.5;

	
		
		for (int i = 0; i < 10; i++) {
			ProfileGenerator p = new ProfileGenerator(false, new Uniform(),new Normal(av, stdev));
			System.out.println(p.generate("A"));
		}
	}

}
