/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package support;

import java.util.ArrayList;
import java.util.List;

import messages.GroupMessage;
import messages.Message;
import messages.Query;
import ants.AntSystem;
import ants.ISSQueryAnt;
import ants.QueryAnt;

public class FloodJob {
	
	private String initiator;
	private List<String> pool;
	private int hitCount;
	private int hitCountISS;
	private int hopsISS;
	private int initialPoolSize;
	private int hops;
	private Message message;
	
	public FloodJob(String initiator, Message msg) {
		this.initiator = initiator;
		this.message = msg;
		this.hitCount = 0;
		this.hops = 0;
		this.hopsISS = 0;
		this.hitCountISS = 0;
		if (msg instanceof GroupMessage) {
			this.pool = new ArrayList<String>();
			this.pool.addAll(NetworkSystem.getNodes());
		} else if (msg instanceof Query) {
			this.pool = NetworkSystem.getMatchingNodes(((Query) msg).getProfile());
		} else {
			this.pool = new ArrayList<String>();
		}
		
		initialPoolSize = pool.size();
	}
	
	public String getQueryIdentifier() {
		return message.getIdentifier();
	}
	
	public synchronized void hop(String from, String to) {
		if (Parameters.NOTIFY)
			System.out.format("Flood hop: %s : %s to %s\n", this.message.getIdentifier(), from, to);
		hops++;
	}
	
	public synchronized void hopOnISS(String from, String to) {
		if (Parameters.NOTIFY)
			System.out.format("Flood ISS hop: %s : %s to %s\n", this.message.getIdentifier(), from, to);
		hopsISS++;
	}
	
	public int getHops() {
		return hops + hopsISS;
	}
	
	public int getHopsOnISS() {
		return hopsISS;
	}
	
	public String getInitiatorIdentifier() {
		return initiator;
	}
	
	public synchronized void hit(String what) {
		if(pool.remove(what))
			hitCount++;
		NetworkSystem.addFound(what);
	}
	
	public synchronized void hitOnISS(String what) {
		if (pool.remove(what))
			hitCountISS++;
		NetworkSystem.addFound(what);
	}
	
	public int getFound() {
		return initialPoolSize - pool.size();
	}
	
	public int getHits() {
		return hitCount + hitCountISS;
	}
	
	public int getHitsOnISS() {
		return hitCountISS;
	}
	
	public float getHitRate() {
		return new Float(getFound()).floatValue() / new Float(initialPoolSize).floatValue();
	}
	
	public void start() {
		if (Parameters.NOTIFY)
			System.out.format("Flood start: %s from %s\n", this.message.getIdentifier(), initiator);
		if (Parameters.ISS_ENABLED) {
			AntSystem.schedule(new ISSQueryAnt(new String(initiator), null, new String(initiator),
					message, Parameters.SEARCH_LIMIT_TTL, false, null));
		} else {
			AntSystem.schedule(new QueryAnt(new String(initiator), null, new String(initiator),
					message, Parameters.SEARCH_LIMIT_TTL, null));
		}
	}

	public int getPoolSize() {
		return initialPoolSize;
	}

}
