/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  BlåtAnt Algorithm implementation
 *  
 *  Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 *  					Pervasive and Artificial Intelligence Research Group
 *  					Department of Informatics DIUF
 *  					University of Fribourg, Fribourg
 *  					Switzerland
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
package support;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Set;

import network.NetworkNodeGen;
import network.RTT;
import remote.IRemoteControllableNetwork;
import services.dss.profile.ResourceProfile;
import util.Graph;
import ants.AntSystem;
import ants.ConstructionLinkAnt;

public class NetworkSystem {

	private static HashMap<String, INetworkNode> nodes;

	private static List<INetworkNode> nodeobj;

	private static int size;
	
	private static Long gtimestamp = 0l;

	public static IRemoteControllableNetwork rcn = null;
	
	public static Hashtable<String, Double> nodeclasses = new Hashtable<String, Double>();

	private static Set<String> foundQueries = new HashSet<String>();

	public static void addNode(INetworkNode n) {
		NetworkNodeGen.grab(n.getIdentifier());
		nodes.put(n.getIdentifier(), n);
		nodeobj.add(n);
		size = nodeobj.size();
		assignClass(n.getIdentifier());
	}
	
	public static void tick() {
		gtimestamp++;
	}
		
	private static void assignClass(String node) {
		double cla = Randomizer.pop();
		nodeclasses.put(node, cla);
	}
	
	private static void removeClass(String node) {
		nodeclasses.remove(node);
	}

	public static boolean isSuperior(String a, String b) {
		return a.hashCode() > b.hashCode();
		/*
		 * if (Integer.parseInt(b) <= Integer.parseInt(a)) return true; else
		 * return false;
		 */
	}

	public static void clear() {
		nodes.clear();
	}

	public static void clearVisit() {
		for (INetworkNode n : nodeobj) {
			n.clearVisit();
		}
	}

	public static boolean existNode(String id) {
		return nodes.containsKey(id);
	}

	public static int getEdgeCount() {
		int count = 0;
		for (INetworkNode n : nodeobj) {
			count = count + n.getNeighborsCount();
		}
		return count;
	}

	public static List<INetworkNode> getNetworkNodes() {
		return nodeobj;
	}

	public static INetworkNode getNode(String id) {
		return nodes.get(id);
	}

	public static int getNodeCount() {
		return size;
	}

	public static Set<String> getNodes() {
		return nodes.keySet();
	}

	public static String getOneNode() {
		return nodeobj.get(nodeobj.size() - 1).getIdentifier();
	}

	public static String getQuasiRandomNode() {
		if (nodeobj.size() <= 10)
			return getReallyRandomNode();
		return nodeobj.subList(0, 10).get((int) (10 * Randomizer.pop()))
				.getIdentifier();
	}

	public static String getReallyRandomNode() {
		return nodeobj.get((int) (nodeobj.size() * Randomizer.pop()))
				.getIdentifier();
	}

	public static void init(int count, int threads) {
		nodes = new HashMap<String, INetworkNode>();
		nodeobj = new ArrayList<INetworkNode>(count);
	}
	
	public synchronized static boolean isNodeAlive(String target) {
		return nodes.keySet().contains(target);
	}

	public synchronized static boolean ping(String target) {
		Stats.ping();
		return nodes.keySet().contains(target);
	}

	public static void removeNode(String n) {
		INetworkNode node = nodes.get(n);
		NetworkNodeGen.release(node.getIdentifier());
		nodes.remove(n);
		nodeobj.remove(node);
		size--;
		removeClass(n);
	}

	public static String getFirstNode() {
		return nodeobj.get(0).getIdentifier();
	}

	public static void initiateUserLink(String a, String b) {
		// First time connection are not urgent (high ttl)
		AntSystem.schedule(new ConstructionLinkAnt(new String(a),
				new String(b), Parameters.USER_LINK_TTL));
	}

	public static void step() {
		for (INetworkNode n : nodeobj) {
			//Algorithm.star.progress();
			n.step();
		}
	}

	public static int getOverloadedCount() {
		int over = 0;
		for (INetworkNode n : nodeobj) {
			if (n.getNeighborsCount() > Parameters.MAX_OPTI_CONNECTIONS)
				over++;
		}
		return over;
	}

	public static void addUserLink(String a, String b, boolean bidi) {
		getNode(a).connect(b);
		if (bidi)
			getNode(b).connect(a);
	}

	public static IRemoteControllableNetwork getNetworkManager() {
		return rcn;
	}

	public static Collection<? extends String> getNodesSample(int count) {
		ArrayList<String> sample = new ArrayList<String>();
		sample.addAll(nodes.keySet());
		Collections.shuffle(sample);
		return sample.subList(0, count);
	}
	
	public static double getAverageCacheSize() {
		if (!Parameters.ISS_ENABLED && !Parameters.ADAPTIVE_ISS) return 0;
		if (nodeobj.size() == 0) return 0;
		double size = 0;
		for (INetworkNode n : nodeobj) {
			size += n.getInformationSystemService().getCacheSize();
		}
		return size / nodeobj.size();
	}
	
	public static long getAverageMergeInterval() {
		if (!Parameters.ISS_ENABLED && !Parameters.ADAPTIVE_ISS) return 0;
		if (nodeobj.size() == 0) return 0;
		long mint = 0;
		for (INetworkNode n : nodeobj) {
				mint += n.getInformationSystemService().getMergeInterval();
		}
		return mint / nodeobj.size();
	}
	
	public static long getAverageProactiveInterval() {
		if (!Parameters.ISS_ENABLED && !Parameters.ADAPTIVE_ISS) return 0;
		if (nodeobj.size() == 0) return 0;
		long pint = 0;
		for (INetworkNode n : nodeobj) {
				pint += n.getInformationSystemService().getProactiveInterval();
		}
		return pint / nodeobj.size();
	}
	
	public synchronized static Graph getGraph() {
		Graph g = new Graph(true);
		for (INetworkNode n : nodeobj) {
			g.addVertex(n.getIdentifier());
			for (String ns : n.getNeighbors()) {
				g.addEdge(n.getIdentifier(), ns);
			}
		}
		return g;
	}

	public static Long getGlobalTimeIter() {
		return gtimestamp;
	}
	
	public static Long getGlobalTimeMs() {
		return gtimestamp * Parameters.MS_PER_ITER;
	}
	
	// Simulate timestamps with some errors (max drift is 50 iterations)
	public static Long getGlobalTimingUnprecise(INetworkNode node) {
		int drift = node.getIdentifier().hashCode() % 50;
		return gtimestamp + drift;
	}

	public static long getRTT(String from, String to) {
		return RTT.getPathWeight(from,to);
	}
	
	public static List<String> getMatchingNodes(ResourceProfile profile) {
		List<String> matching = new ArrayList<String>();
		for (String b : nodes.keySet()) {
			ResourceProfile bn = getNode(b).getProfile();
			if (bn.matches(profile))
				matching.add(new String(b));
		}
		return matching;
	}

	public static boolean isFound(String identifier) {
		synchronized(foundQueries) {
			return foundQueries.contains(identifier);
		}
	}
	
	public static void addFound(String queryid) {
		synchronized(foundQueries) {
			foundQueries.add(queryid);
		}
	}

	public static Graph getGraph(boolean b) {
		Graph g = new Graph(b);
		for (INetworkNode n : nodeobj) {
			g.addVertex(n.getIdentifier());
			for (String ns : n.getNeighbors()) {
				g.addEdge(n.getIdentifier(), ns);
			}
		}
		return g;
	}


}
