/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package support;

import java.lang.reflect.Field;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;




public class Parameters {
	
	/* DSS Stuff */
	public static boolean DSS = false;
	public static int DSS_JOB_MIN_MATCHES = 2;
	public static Double DSS_DRIFT = 0.1;
	public static int DSS_MAX_RETRIES = 0;
	public static int DSS_INFORM_TTL = 6;
	public static int DSS_INFORM_FFW = 3;
	public static int DSS_QUERY_TRACK_SIZE = 2048;
	public static boolean DSS_VISUAL_SCHEDULER = false;
	public static long INFORM_ACCEPT_THRESHOLD_MS = 180000; // 30 minutes difference

	public static int INFORM_LAST_N = 2;
	public static double FIFO_PROBABILITY = 0.5;
	
	public static boolean CONN_ERROR = false;
	
	
	public static boolean BAD_DRIFT_ONLY = false;

	public static boolean QUERY_PATH_TRACKING = false;
	
	public static boolean ADAPTIVE_ISS = false;

	public static long MIN_ITER_MILLIS = -1;
	
	public static boolean SMARTGRID_PROXY = false;

	public static boolean SEQ_COMPUTE = false;
	public static String PREFIX = null;
	
	
	
	
	

	/* Time depended parameters */
	public static int REMOVE_NODE_INTERVAL_MS = -1;
	public static int LEAVE_NODE_INTERVAL_MS = -1;
	public static int ADD_NODE_INTERVAL_MS = -1;
	public static int SEARCH_NODE_INTERVAL_MS = -1; 
	public static int ADD_NODE_START_MS = -1;
	public static int REMOVE_NODE_START_MS = -1;
	public static int LEAVE_NODE_START_MS = -1;
	public static int SEARCH_NODE_START_MS = -1;
	public static int REMOVE_NODE_STOP_MS = -1;
	public static int ADD_NODE_STOP_MS = -1;
	public static int LEAVE_NODE_STOP_MS = -1;
	public static int SEARCH_NODE_STOP_MS = -1;
	public static long CONNECTION_RETRY_INTERVAL_MS = 10000;
	public static long INFORM_INTERVAL_MS = 600000; // in milliseconds
	public static long ACCEPT_OFFER_WAIT_MS = 50000; // in milliseconds
	public static int PACKET_LOSS_START_MS = -1;
	public static Integer PACKET_LOSS_STOP_MS = -1;
	public static Integer PACKET_SLOW_START_MS = -1;
	public static Integer PACKET_SLOW_STOP_MS = -1;
	public static long REQUEST_MIN_INTERVAL_MS = 100; // How many iterations per
	public static long GAMMA_DECAY_MS = 30000; // how many iterations to go form default to minimum
	public static long BETA_DECAY_MS = 60000; // how many iterations to go form default to minimum
	public static int NS_MERGE_INTERVAL_MS = 600000;
	public static int GNUTELLA_PING_MS = 600000;
	public static long ISS_MERGE_INTERVAL_MS = 250000;
	public static long ISS_PROACTIVE_QUERY_INTERVAL_MS = 2500000;
	public static long ISS_MERGE_START_MS = -1;
	public static long ISS_PROACTIVE_QUERY_START_MS = -1;
	public static long DSS_MAX_JOBTIME_MS = 14400000; // 30 min
	public static long DSS_MIN_JOBTIME_MS = 3600000; // 5 min
	public static long DSS_MIN_DEADLINE_MS = 18000000l; // 5 hour
	public static long DSS_MAX_DEADLINE_MS = 36000000l; // 10 hours ... average in 7.5 hours
	public static long REPLICATION_ANT_INTERVAL_MS = 600000; // 10 min
	public static long CACHE_TIME_MS = 1200000; // 20 min with rtt 100
	public static long ALPHA_MAX_AGE_MS = 300000;
	public static long UPDATE_INTERVAL_OMEGA_CON_MS = 2500;
	public static long UPDATE_INTERVAL_OMEGA_DIS_MS = 2500;
	
	
	
	
	
	
	
	public static int SEARCH_NODE_COUNT = -1;
	public static int FORCE_ACCEPT_ON_TTL = 10;
	public static int SEARCH_LIMIT_FFW = -1;
	public static int SEARCH_LIMIT_TTL = -1;
	public static boolean ADD_NODE_RANDOM_HOOK = false;
	public static int ADD_COUNT = -1;
	public static int REM_COUNT = -1;
	public static int LEAVE_COUNT = -1;

	/* Debugging */
	public static boolean DEBUG = false;
	public static boolean NOTIFY = false;

	/* Fail resilience */
	public static int EMERGENCY_TTL = 0; /* Emergency, urgent */


	/* Network problems */
	public static double PACKET_SLOW_PROB = 0;
	public static double PACKET_LOSS_PROB = 0;


	/* Alpha table */
	
	public static int ALPHA_MAX_SIZE = 20;


	/* DAnt */
	public static int ANT_LEN = 15;
	public static float RANDOM_WALK_PROBABILITY = 0.5f;

	/* UserLinkAnt */
	public static int USER_LINK_TTL = 20; /* Non emergency connections */
	public static int LEAVE_LINK_TTL = 0; /* Leave, urgent */

	/* Evaluation options */
	
	public static int D = 5;
	public static int K = 1;

	/* Pheromone */
	public static Double DEF_INCREMENT = 0.1; // delta
	public static Double INITIAL_CONCENTRATION = 1.0;




	/* Node options */
	public static int MAX_CONNECTIONS = -1;
	public static int MAX_OPTI_CONNECTIONS = -1;

	/* Ant respawn options */
	public static int ANT_MAX_AGE = 25;
	public static Double ANT_BIRTH_PROBABILITY = 0.05;
	public static long IOTA_MS = 100000;

	/* Shortcuts (fixed as new model from the thesis) */
	public volatile static int connthreshold = -1;
	public volatile static int deconnthreshold = -1; ; 

	/* Others */
	public static long RANDOM_SEED = 81;
	public static boolean COMPLETE_NS_LCC = false;
	public static boolean SPIDER_WEB_CRASH_RECOVERY = false;

	/* Random walkers */
	public static int WALKER_CALL_BACK = 6;

	/* Invalid (now) */
	public static boolean DISABLE_BLATANT = false;
	public static Boolean BALANCING = false;
	public static Integer FLOODTTL = 12;

	/* Newscast */
	public static int NS_CACHE_SIZE = 20;
	
	
	

	/* Gnutella */
	public static int GNUTELLA_NEIGHBORS = 10;
	public static int GNUTELLA_TTL = 7;
	public static int GNUTELLA_FW = 4;
	
	public static String INITIAL = "";

	public static int COMPUTE_INTERVAL = 200;
	public static int DISTANCE_INTERVAL = -1;
	public static int INTERVAL = 500;
	public static int SCREEN_INTERVAL = 20;

	public static int MAX_ITER = 2500;
	public static Float CPUFACTOR = 1f;
	public static int CPUCOUNT = -1;

	public static int COMPUTE_FROM = 0;
	public static boolean REAL_STAR = false;
	public static boolean LOG_COMPUTE = false;
	public static Float LOG_SCALE = 1.5f;
	
	/* CroissAnt 2nd level overlay */
	public static int ISS_CACHE_SIZE = 10;

	public static int ISS_PROACTIVE_QUERY_TTL = 5;
	public static int ISS_PROACTIVE_QUERY_FFW = 3;
	public static boolean ISS_ENABLED = false;
	public static int ISS_QUERY_TTL = 3;
	public static int ISS_QUERY_FFW = 10;
	public static boolean NONSTOP_SEARCH = false;

	public static int SHIFT_INTERVAL = 100;

	public static int SAMPLE_COMPUTE = -1;

	public static boolean COVERAGE = false;

	public static boolean NONLIMIT_SEARCH = false;

	public static boolean ISS_DISJOINT_OVERLAYS = true;

	public static boolean DETAIL_DIAMETER = false;

	public static Boolean ALWAYS_FORWARD_LEAFS = false;

	public static boolean RTT_MIGRATION = false;

	public static String UNDERLYING_NETWORK = "";

	public static int MS_PER_ITER = 1;
	public static double MS_PER_ITER_DOUBLE = 1;
	
	public static boolean PROFILE_SIMILARITY = false;
	
	
	public static boolean DSS_EDF = false;
	public static boolean DSS_AR = false;
	public static boolean DSS_AR_FAIR = false;
	public static boolean DSS_AR_FAIR_POOL = false;
	public static int DSS_AR_FAIR_MAX_POOL_SIZE = 4;
	
	public static boolean SKIP_ARCH_MATCH = false;
	public static boolean SKIP_OS_MATCH = false;
	public static boolean SKIP_VER_MATCH = false;
	public static boolean SKIP_MEM_MATCH = false;
	public static boolean SKIP_DISK_MATCH = false;
	
	public static boolean NO_CPUSPEED = false;
	
	public static String SWF_FILE = null;
	public static Integer SWF_SCALE = 1;
	public static String DSS_BROKER = null;
	public static boolean RTT_GENERATOR = false;
	public static int QUERY_CACHE_SIZE = 128;
	
	
	public static int REPLICATION_CACHE_SIZE = 5;
	public static boolean UNIFORM_PROFILES = false;
	
	
	public static boolean PROFILE_CACHE = false;
	public static boolean ONE_HOP_REPLICATION = false;
	
	public static boolean GIRTH = false;
	public static boolean DEGREE = false;
	public static boolean CLUSTERING = false;
	public static boolean GNUSTATS = false;
	public static long NODE_RESOLUTION_INTERVAL_MS = 250;
	public static long NODE_RESOLUTION_INTERVAL_ITER = -1;
	public static int TRAIL_DECAY_STEP = 5;
	public static boolean DO_MULTITHREAD = true;
	public static long DO_MULTITHREAD_PROBE = 1000;
	public static boolean NO_RANDOM_CACHE = false;
	public static boolean NO_DISCONNECTION = false;

	public static void print() {
		
		List<String> names = new ArrayList<String>();
		for (Field f : Parameters.class.getFields()) {
			names.add(f.getName());
		}
		Collections.sort(names);
		
		for (String fn : names) {
			try {
				Field f = Parameters.class.getField(fn);
				Object value = f.get(null);
				System.out.format("%s = %s %s\n", f.getName(), value, f.getName().endsWith("_MS") ? formatTime2(value) :  "");
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (SecurityException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NoSuchFieldException e) {
				e.printStackTrace();
			}
		}

		/* Shortcuts */
		System.out.format("---------------------------------------------\n");
		System.out.format("Threshold for Rule 1 (2D-1): %d\n", connthreshold);
		System.out.format("Threshold for Rule 2 (2D-3): %d\n", deconnthreshold);
		System.out.format("Expected upper bound: %d\n", connthreshold - 1);
		System.out.format("Expected lower bound: %d\n", connthreshold / 2);
		System.out.format("---------------------------------------------\n");
		if (ANT_LEN < connthreshold)
			System.out
					.println("Warning: Either the threshold for Rule 1 is too large or ANT_LEN is too small");

	}
	
	private static String formatTime2(Object value) {
		long millis;
		if (value instanceof Integer) {
			millis = (Integer) value;
		} else {
			millis = (Long) value;
		}
		long hours = millis / 3600000l;
		if (hours > 1000) {
			return DateFormat.getDateTimeInstance(DateFormat.LONG,
					DateFormat.LONG).format(millis);
		}
		long minutes = (millis - hours * 3600000) / 60000;
		long seconds = (millis - (minutes * 60000 + hours * 3600000)) / 1000;
		long ms = millis - (minutes * 60000 + hours * 3600000 + seconds * 1000);
		long iters = Math.round(millis / Parameters.MS_PER_ITER_DOUBLE);
		return String.format("(%dh %dm %ds %dms / %d iterations [%d iter/ms])", hours, minutes, seconds, ms, iters, MS_PER_ITER);
	}


}
