/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package syncutil;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import support.NetworkSystem;

public class AdjacencyMap {

	public int[][] map;
	public int size = 0;

	public AdjacencyMap() {
		size = NetworkSystem.getNodeCount();
		map = new int[size][size];
		for (int i = 0; i < size; i++) {
			for (int j = 0; j < size; j++) {
				map[i][j] = 0;
			}
		}
	}

	public void update() {
		size = NetworkSystem.getNodeCount();
		List<String> nodes = new ArrayList<String>(size);
		nodes.addAll(NetworkSystem.getNodes());
		Collections.sort(nodes);
		for (int i = 0; i < size; i++) {
			for (int j = 0; j < size; j++) {
				Collection<String> ins = NetworkSystem.getNode(nodes.get(i))
						.getNeighbors();
				map[i][j] = ins.contains(nodes.get(j)) ? 1 : 0;
			}
		}
	}

	public int distanceCopyFrom(AdjacencyMap b) {
		int d = 0;
		if (b.size != size)
			return -1;
		for (int i = 0; i < size; i++) {
			for (int j = 0; j < size; j++) {
				if (b.map[i][j] != map[i][j])
					d++;
				map[i][j] = b.map[i][j];
				b.map[i][j] = 0;
			}
		}
		return d;
	}

}
