/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package syncutil;

import java.util.Collection;

import support.INetworkNode;
import support.NetworkSystem;

public class Clustering {
	
	private double clustering = 0.0f;
	private boolean computed = false;
	
	public void computeClustering() {
		
		double totalClustering = 0f;
		
		for (String n : NetworkSystem.getNodes()) {
			totalClustering += localClustering(n);
		}
		
		clustering = totalClustering / NetworkSystem.getNodeCount();
		computed = true;
	}
	
	public void printClustering() {
		
		if (computed)
			System.out.println("\t!Clustering: "
					+ clustering);
		else
			System.out.println("\t?Clustering: "
					+ clustering);

		computed = false;
	}

	private double localClustering(String n) {
		double counter = 0;
		INetworkNode node = NetworkSystem.getNode(n);
		Collection<String> neighbors = node.getNeighbors();
		double neighborsCount = new Double(neighbors.size());
		double possibleConnections = (neighborsCount * (neighborsCount - 1.0f)) / 2.0f;
		for (String neighbor : neighbors) {
			INetworkNode nn = NetworkSystem.getNode(neighbor);
			if (nn != null) {
				Collection<String> nneigh = nn.getNeighbors();
				for (String ns : nneigh) { // If the neighbor's neighbor is in the neighbors of n count as 1
					if (neighbors.contains(ns)) {
						counter += 1;
					}
				}
			}
		}
		return counter / possibleConnections;
	}

}
