/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package syncutil;

import java.util.HashMap;
import java.util.Map;

import support.INetworkNode;
import support.NetworkSystem;

public class DegreeDistribution {
	
	private Map<Integer,Long> degrees = new HashMap<Integer,Long>();
	private Long totalNodes = 0l;
	private boolean computed = false;
	
	public void computeDegreeDistribution() {
		degrees.clear();
		
		totalNodes = new Long(NetworkSystem.getNodeCount());
		for (String n : NetworkSystem.getNodes()) {
			INetworkNode node = NetworkSystem.getNode(n);
			int neighbors = node.getNeighborsCount();
			addNode(neighbors);
		}
		
		computed = true;
	}
	
	private void addNode(Integer d) {
		Long value = 1l;
		if (degrees.containsKey(d)) {
			value = new Long(degrees.get(d)+1l);
		}
		degrees.put(d, value);
	}



	public void printDegree() {
		
		if (computed)
			System.out.format("\t!Degrees (%d): ",totalNodes);
		else
			System.out.format("\t?Degrees (%d): ",totalNodes);

		for (Integer d : degrees.keySet()) {
			System.out.format("d(%d)=%d , ", d, degrees.get(d));
		}
		
		System.out.println();
		computed = false;
	}
	
}
