/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
package syncutil;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import support.INetworkNode;
import support.NetworkSystem;
import support.Parameters;

public class Girth {

	private int girth = Integer.MAX_VALUE;
	private Map<Integer, Integer> cycles = new HashMap<Integer, Integer>();
	private boolean computed = false;

	public void computeGirth() {		
		cycles.clear();
		girth = Integer.MAX_VALUE;
		for (String n : NetworkSystem.getNodes()) {
			distancesFromSource(n);
		}
		computed = true;
	}

	private void distancesFromSource(String source) {
		Map<String, Integer> visited = new Hashtable<String, Integer>();
		Set<String> toVisit = new HashSet<String>();
		Set<String> visiting = new HashSet<String>();
		int distance = 1;
		boolean shouldStop = false;
		
		toVisit.add(source);
		visited.put(source, new Integer(0));
		
		while (!toVisit.isEmpty()) {
			visiting.clear();
			visiting.addAll(toVisit);
			toVisit.clear();
			for (String enne : visiting) {
				String n = new String(enne);
				INetworkNode node = NetworkSystem.getNode(n);
				if (node != null) {
					node.acquire();
					for (String k : node.getNeighbors()) { // For all neighbors
						INetworkNode u = NetworkSystem.getNode(k);
						if (u != null)
							if (!Parameters.COMPLETE_NS_LCC
									|| u.getNeighbors().contains(n))
								if (!visited.containsKey(k)) { // If not visited visit
									visited.put(k, new Integer(distance)); // Set its distance								
									toVisit.add(new String(k));
								} else {
									if (visiting.contains(k)) { // Is to be visited at this step
										int cyclelength =  2 * distance - 1;
										if ((3 <= cyclelength)) {
											/*if (cyclelength == 3) {
												System.out.println("Cycle of length 3 "+ n+" via "+ k);
											}*/
											addCycle(cyclelength); // I count, he will count
											if (cyclelength < girth)
												girth = cyclelength;
											shouldStop = true;
										}
									} else if (toVisit.contains(k)) { // Will be visited next
										int cyclelength =  2 * distance;
										if ((3 <= cyclelength)) {
											/*if (cyclelength == 3) {
												System.out.println("Semi Cycle of length 3 "+ n+" via "+ k);
											}*/
											addCycle(cyclelength); // I should count for both
											addCycle(cyclelength);
											if (cyclelength < girth)
												girth = cyclelength;
											
											shouldStop = true;
										}
										
									}
								}
					}
					node.release();
				}

			}
			if (shouldStop) break;
			distance++;
		}
	}

	private void addCycle(Integer distance) {
		Integer newd = 1;
		if (cycles.containsKey(distance)) {
			newd = cycles.get(distance) + 1;
		}
		cycles.put(distance, newd);
	}

	public void printGirth() {
		List<Integer> clengths = new ArrayList<Integer>();
		clengths.addAll(cycles.keySet());
		Collections.sort(clengths);

		if (computed)
			System.out.format("\t!Girth: "
					+ (girth == Integer.MAX_VALUE ? "n/a" : girth));
		else
			System.out.format("\t?Girth: "
					+ (girth == Integer.MAX_VALUE ? "n/a" : girth));

		for (Integer c : clengths) {
			int ccount = cycles.get(c) / (2*2*c);
			ccount = ccount == 0 ? 1 : ccount;
			System.out.format(" , C(" + c + ")= " + ccount);
		}

		System.out.println();
		computed = false;
	}

}
