/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  BlåtAnt Algorithm implementation
 *  
 *  Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 *  					Pervasive and Artificial Intelligence Research Group
 *  					Department of Informatics DIUF
 *  					University of Fribourg, Fribourg
 *  					Switzerland
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
package syncutil;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

public class SyncGraph {

	private HashMap<String, Set<String>> vertexes;
	private boolean directed = false;

	public SyncGraph(boolean directed) {
		this.directed = directed;
		vertexes = new HashMap<String, Set<String>>();
	}

	public void addEdge(String from, String to) {
		addVertex(from);
		addVertex(to);
		addSuccessor(from, to);
		if (!directed)
			addSuccessor(to, from);
	}

	private boolean addSuccessor(String id, String succ) {
		Set<String> adj = vertexes.get(id);
		return adj.add(succ);
	}

	public void addVertex(String id) {
		if (!vertexes.containsKey(id))
			vertexes.put(id, new HashSet<String>());
	}

	public synchronized Set<String> getSuccessors(String id) {
		return vertexes.get(id);
	}

	public void removeEdge(String from, String to) {
		Set<String> adj = vertexes.get(from);
		adj.remove(to);
		if (!directed) {
			adj = vertexes.get(to);
			adj.remove(from);
		}
	}

	public void removeVertex(String id) {
		vertexes.remove(id);
		for (Set<String> adj : vertexes.values()) {
			adj.remove(id);
		}
	}

	public Set<String> vertexSet() {
		return vertexes.keySet();
	}

	public void removeNonBidirectionalEdges() {
		for (String v : vertexes.keySet()) {
			List<String> invalid = new LinkedList<String>();
			Set<String> succ = getSuccessors(v);
			for (String k : succ) {
				if (!getSuccessors(k).contains(v)) {
					invalid.add(k);
				}
			}
			succ.removeAll(invalid);
		}
	}

	public boolean isDirected() {
		return directed;
	}

}
