/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  BlåtAnt Algorithm implementation
 *  
 *  Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 *  					Pervasive and Artificial Intelligence Research Group
 *  					Department of Informatics DIUF
 *  					University of Fribourg, Fribourg
 *  					Switzerland
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
package util;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class Dijkstra {

	/*
	 * Dijkstra, shortest path Adapted from the Wikipedia, The Free Encyclopedia
	 * (http://en.wikipedia.org/wiki/Dijkstra%27s_algorithm)
	 */

	private static int INF = 100000000;
	
	public static List<String> getShortestPath(Graph g, String source,
			String target, String tabu) {
		if (target.equals(tabu)) return new LinkedList<String>();
		Map<String, Integer> dist = new HashMap<String, Integer>(100);
		Map<String, String> previous = new HashMap<String, String>(100);
		Set<String> Q = new HashSet<String>();
		for (String v : g.vertexSet()) {
			if (!v.equals(tabu))
				dist.put(v, INF);
		}
		dist.put(source, 0);
		Q.addAll(g.vertexSet());
		Q.remove(tabu);
		while (!Q.isEmpty()) {
			String u = popMin(Q, dist);
			if (u.equals(target))
				break;
			for (String v : g.getSuccessors(u)) {
				if (v.equals(tabu)) continue;
				int current = dist.get(v);
				int upd = dist.get(u) + 1;
				if (upd < current) {
					dist.put(v, upd);
					previous.put(v, u);
				}
			}
		}

		List<String> path = new LinkedList<String>();
		String u = target;
		while (previous.containsKey(u)) {
			path.add(0, u);
			u = previous.get(u);
		}
		path.add(0, source);
		
		if (source.equals("1") && target.equals("8") && tabu.equals("14")) {
			System.out.println(path);
		}
		return path;
	}

	public static List<String> getShortestPath(Graph g, String source,
			String target) {
		Map<String, Integer> dist = new HashMap<String, Integer>(100);
		Map<String, String> previous = new HashMap<String, String>(100);
		Set<String> Q = new HashSet<String>();
		for (String v : g.vertexSet()) {
			dist.put(v, INF);
		}
		dist.put(source, 0);
		Q.addAll(g.vertexSet());
		while (!Q.isEmpty()) {
			String u = popMin(Q, dist);
			if (u.equals(target))
				break;
			for (String v : g.getSuccessors(u)) {
				int current = dist.get(v);
				int upd = dist.get(u) + 1;
				if (upd < current) {
					dist.put(v, upd);
					previous.put(v, u);
				}
			}
		}

		List<String> path = new LinkedList<String>();
		String u = target;
		while (previous.containsKey(u)) {
			path.add(0, u);
			u = previous.get(u);
		}
		path.add(0, source);
		return path;
	}

	public static void main(String[] args) {
		unitTest();
	}

	private static String popMin(Set<String> q, final Map<String, Integer> d) {
		Set<String> e = new HashSet<String>();
		e.addAll(q);
		String m = Collections.min(e, new Comparator<String>() {
			public int compare(String o1, String o2) {
				return d.get(o1).compareTo(d.get(o2));
			}
		});
		q.remove(m);
		return m;
	}

	public static void unitTest() {
		Graph g = new Graph(false);
		g.addEdge("0", "1");
		g.addEdge("0", "2");
		g.addEdge("1", "3");
		g.addEdge("2", "3");
		g.addEdge("0", "4");
		g.addEdge("1", "4");
		g.addEdge("3", "4");
		g.addEdge("4", "5");

		// Unconnected part
		g.addEdge("7", "9");
		g.addEdge("7", "8");
		g.addEdge("8", "10");

		System.out.println("2->4 =>" + getShortestPath(g, "2", "4"));
		System.out.println("5->0 =>" + getShortestPath(g, "5", "0"));
		System.out.println("1->5 =>" + getShortestPath(g, "1", "5"));
		System.out.println("7->10 =>" + getShortestPath(g, "7", "10"));
		System.out.println("1->10 =>" + getShortestPath(g, "1", "10"));
	}

}
