/*******************************************************************************
 * BlåtAnt Algorithm Evaluation Platform
 * 
 * Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 * 					Pervasive and Artificial Intelligence Research Group
 * 					Department of Informatics DIUF
 * 					University of Fribourg, Fribourg
 * 					Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************/
/**
 *  BlåtAnt Algorithm implementation
 *  
 *  Copyright (C) 2008  Amos Brocco <amos.brocco@unifr.ch>
 *  					Pervasive and Artificial Intelligence Research Group
 *  					Department of Informatics DIUF
 *  					University of Fribourg, Fribourg
 *  					Switzerland
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
package util;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;

public class WeightedGraph {

	private TreeMap<String, Set<String>> vertexes;
	private TreeMap<String, Long> weights;
	private boolean directed = false;

	public WeightedGraph(boolean directed) {
		this.directed = directed;
		vertexes = new TreeMap<String, Set<String>>();
		weights = new TreeMap<String, Long>();
	}
	
	private void addWeight(String from, String to, Long weight) {
		weights.put(String.format("%s->%s", from, to), weight);
	}
	
	private void removeWeight(String from, String to) {
		weights.remove(String.format("%s->%s", from, to));
	}
	
	public Long getWeight(String from, String to) {
		String key = String.format("%s->%s", from, to);
		return weights.get(key);
	}

	public void addEdge(String from, String to, long weight) {
		addVertex(from);
		addVertex(to);
		addSuccessor(from, to, weight);
		if (!directed)
			addSuccessor(to, from, weight);
	}

	private boolean addSuccessor(String id, String succ, long weight) {
		Set<String> adj = vertexes.get(id);
		addWeight(id, succ, weight);
		return adj.add(succ);
	}

	public void addVertex(String id) {
		if (!vertexes.containsKey(id))
			vertexes.put(id, new HashSet<String>());
	}

	public Set<String> getSuccessors(String id) {
		if (vertexes.containsKey(id))
			return vertexes.get(id);
		else
			return new HashSet<String>();
	}

	public void removeEdge(String from, String to) {
		Set<String> adj = vertexes.get(from);
		adj.remove(to);
		removeWeight(from, to);
		if (!directed) {
			adj = vertexes.get(to);
			adj.remove(from);
			removeWeight(to, from);
		}
	}

	public void removeVertex(String id) {
		vertexes.remove(id);
		for (Set<String> adj : vertexes.values()) {
			adj.remove(id);
		}
	}

	public Set<String> vertexSet() {
		return vertexes.keySet();
	}

	public void removeNonBidirectionalEdges() {
		for (String v : vertexes.keySet()) {
			List<String> invalid = new LinkedList<String>();
			Set<String> succ = getSuccessors(v);
			for (String k : succ) {
				if (!getSuccessors(k).contains(v)) {
					invalid.add(k);
				}
			}
			succ.removeAll(invalid);
		}
	}

	public boolean isDirected() {
		return directed;
	}

	public int getVertexCount() {
		return vertexes.keySet().size();
	}

}
